import BN from 'bn.js';
import { MessageAccountKeys, MessageHeader, ParsedMessageAccount, MessageCompiledInstruction, PartiallyDecodedInstruction, PublicKey, CompiledInnerInstruction, ParsedInnerInstruction, TransactionError } from '@solana/web3.js';

type ConvertOptions = {
    displayMode?: "base58" | "base64";
};
declare const convertBNToNumberInObject: (object: Record<string, any>, options?: ConvertOptions) => Record<string, any>;

declare const decodeBase58: (base58Str: string) => Uint8Array;
declare const encodeBase58: (bytes: Uint8Array | Buffer) => string;

declare function getStackFromLogs(logs: string[]): Generator<ExecutionStack>;
type ExecutionStack = {
    parentProgram?: string;
    childProgram?: string;
    depth: number;
    parentIndex: number;
    innerIndex?: number;
};

declare function displaySmallNumber(amount?: number | bigint): string;
declare function countNumberOfDigits(amount?: number | bigint): number;

type TruncateStringOptions = "left" | "center" | "right" | "none";
type TransactionMessageHeader = {
    /**
     * The number of signatures required for this message to be considered valid. The
     * signatures must match the first `numRequiredSignatures` of `accountKeys`.
     */
    numRequiredSignatures: number;
    /** The last `numReadonlySignedAccounts` of the signed keys are read-only accounts */
    numReadonlySignedAccounts: number;
    /** The last `numReadonlySignedAccounts` of the unsigned keys are read-only accounts */
    numReadonlyUnsignedAccounts: number;
};

declare const capitalizeFirstLetter: (string: string) => string;
declare const splitAndCapitalize: (str: string) => string;
declare const camelCaseToStartCase: (string: string) => string;
declare const toCamelCase: (string: string) => string;
declare const formatBytes: (bytes: number, decimals?: number) => string;
declare const formatNumberWithCommas: (number: number) => string;
declare const truncateString: (inputString: string, length: number, truncatePosition: TruncateStringOptions) => string;
/**
 * This function takes in an input number in string or number data type and returns a result string that represents the number without the scientific notation (e.g. 0.00000001 instead of 1e-8)
 * @param inputNumber
 * @param decimals
 */
declare const preventScientificNotationForNumber: (inputNumber: number | string, decimals?: number) => string;
/**
 * This function takes in an input number and the size of the decimal places to pad with zeros
 * and returns a result string that represents the number with the padded zeros to the end.
 *
 * Example:
 * ```
 * const result = addPaddedZeros(1.1, 4);
 * console.log(result); // 1.1000
 * ```
 *
 * @param num
 * @param size
 * @returns
 */
declare const addPaddedZerosToEnd: (num: number | string, size: number) => string;

declare const convertTSToRelativeTime: (timestamp: number | null | undefined, format?: boolean) => string;
declare const convertTSToFormat: (timestamp: number | null | undefined, format: string) => string;

declare const LAMPORTS_PER_SOL = 1000000000;
declare function lamportsToSol(lamports: number | BN | undefined | string): number;
declare function solToLamports(sol: number): number;
declare function numberWithSeparator(s: string | number): string;
declare function lamportsToSolString(lamports: number | BN, maximumFractionDigits?: number): string;
declare const solInStringWithoutScientificNotation: (sol: number) => string;
declare function SolBalance(lamports: number | BN, maximumFractionDigits?: number): string;
declare function normalizeTokenAmountAsString(rawAmount: number | string | bigint, decimals?: number): string;
/**
 * A version of normalizeTokenAmountAsString that only deals with the string type for both input and output.
 */
declare function normalizeTokenAmountInStringAsString(rawAmount: string, decimals?: number): string;

/**
 * Maps the key type for non-parseable account keys
 * @param accountKeys
 * @param keysHeader
 * @param serializeTx
 * @returns {ParsedMessageAccount[]}
 */
declare const getAccountKeyTypeFromKeys: (accountKeys: MessageAccountKeys, keysHeader: MessageHeader, serializeTx?: boolean) => ParsedMessageAccount[];
/**
 * Maps account hashes from the instruction accounts index, also retrieves a list of unique program hashes used in the transaction
 *
 * @param instructions - The compiled instructions of a transaction.
 * @param accountKeys - The parsed message accounts of a transaction.
 * @returns An array containing the partially decoded instructions and an array of unique program hashes used in the transaction.
 */
declare const mapIxAccountIndexes: (instructions: MessageCompiledInstruction[], accountKeys: ParsedMessageAccount[]) => [PartiallyDecodedInstruction[], string[]];
declare const mapIxAccountIndexesV2: (instructions: MessageCompiledInstruction[], accountKeys: PublicKey[]) => [PartiallyDecodedInstruction[], string[]];
/**
 * Maps account hashes in inner instructions from the accounts index, also populates the parser hashmap with unique program hashes
 * @param innerInstructions - The compiled inner instructions of a transaction from the metadata
 * @param accountKeys - The parsed message accounts of a transaction
 * @param uniqueProgramHashes - The unique program hashes used in the transaction that's compiled from the parent instructions
 * @returns An array containing the partially decoded instructions and an array of unique program hashes used in the transaction
 */
declare const mapInnerIxAccountIndexes: (innerInstructions: CompiledInnerInstruction[], accountKeys: ParsedMessageAccount[], uniqueProgramHashes: string[]) => [ParsedInnerInstruction[], string[]];
/**
 * Convert logs from meta to event map
 * @param {string[]} logs - Logs from meta
 */
declare const convertLogsToEventMap: (logs: string[]) => Map<string, string[]>;
declare const convertLogsToErrorMap: (logs: string[], txError?: TransactionError | null) => Map<string, string[]>;

declare const clusterPath: (pathname: string, queryString: string) => string;
declare const checkValidHttpUrl: (url: string) => boolean;

declare const WRAPPED_SOL_MINT = "So11111111111111111111111111111111111111112";

export { type ExecutionStack, LAMPORTS_PER_SOL, SolBalance, type TransactionMessageHeader, type TruncateStringOptions, WRAPPED_SOL_MINT, addPaddedZerosToEnd, camelCaseToStartCase, capitalizeFirstLetter, checkValidHttpUrl, clusterPath, convertBNToNumberInObject, convertLogsToErrorMap, convertLogsToEventMap, convertTSToFormat, convertTSToRelativeTime, countNumberOfDigits, decodeBase58, displaySmallNumber, encodeBase58, formatBytes, formatNumberWithCommas, getAccountKeyTypeFromKeys, getStackFromLogs, lamportsToSol, lamportsToSolString, mapInnerIxAccountIndexes, mapIxAccountIndexes, mapIxAccountIndexesV2, normalizeTokenAmountAsString, normalizeTokenAmountInStringAsString, numberWithSeparator, preventScientificNotationForNumber, solInStringWithoutScientificNotation, solToLamports, splitAndCapitalize, toCamelCase, truncateString };
